create type query_match_score_t as object 
( score number(20,10) 
, weight number(5,2) 
, criteria varchar2(2000) 
)
/

create type query_match_score_table_t as table of query_match_score_t
/




create or replace function number_range_score
( p_value in number
, p_target in number
, p_low_limit in number 
, p_upper_limit in number
) return number
is
  l_score number(5,4):= case p_value when p_target then 1 else 0 end;
begin
  if p_value< p_target and p_value >= p_low_limit
  then 
    l_score := (p_value - p_low_limit)/(p_target - p_low_limit);
  elsif p_value> p_target and p_value <= p_upper_limit
  then
      l_score := (p_upper_limit - p_value)/(p_upper_limit - p_target);
  end if;
  return l_score;
end;

  
  
  
select number_range_score( 950, 1500, 1000, 2000)
from   dual
/
  
  
select empno
,      sum(score * weight) weighted_score
,      cast(collect(query_match_score_t( score, weight,criteria)) as query_match_score_table_t) criteria
from   ( select empno
         ,      number_range_score(sal, 1500, 1000, 2000) score
         ,      2 weight
         ,      'Salary must be as close to 1500 as possible, and at least between 1000 and 2000' criteria
         from   emp
         union all
         select empno
         ,      number_range_score(extract (year from hiredate), 1982, 1982, 1983) score
         ,      1 weight
         ,      'Employee is ideally hired in 1982; a hiredate in 1983 is also somewhat valuable'
         from   emp
       )
group
by     empno
order
by     weighted_score desc


create or replace type string_table as table of varchar2(4000)



select empno
,      (select ename||' - '||job||' - '||hiredate||' - '||sal from emp where empno = scores.empno) details
,      weighted_score
,      ( select cast(collect( substr(criteria,1,105)||': '||score||' ('||weight||')') as string_table)
         from table(criteria)
         where score > 0) criteria
from   ( select empno
         ,      sum(score * weight) weighted_score
         ,      cast(collect(query_match_score_t( score, weight,criteria)) as query_match_score_table_t) criteria
         from   (
                 select empno
                 ,      0 score
                 ,      0 weight
                 ,      'base' criteria
                 from   emp e
                 union all
                 select empno
                 ,      number_range_score(sal, 1500, 1000, 2000) score
                 ,      2 weight
                 ,      'Salary must be as close to 1500 as possible, and at least between 1000 and 2000'  criteria
                 from   emp
                 union all
                 select empno
                 ,      number_range_score(extract (year from hiredate), 1982, 1982, 1983) score
                 ,      1 weight
                 ,      'Employee is ideally hired in 1982; a hiredate in 1983 is also somewhat valuable'
                 from   emp
                )
         group
         by     empno
         order
         by     weighted_score desc
       ) scores
where  rownum < 6


select empno
,      case job when 'CLERK' then 1 when 'ANALYST' then 0.6 when 'SALESMAN' then 0.4 end score
,      1.5 weight
,      'We prefer CLERKs; ANALYST scores 60% and SALESMAN is 40%'
from   emp



select empno
,      (select ename||' - '||job||' - '||hiredate||' - '||sal from emp where empno = scores.empno) details
,      weighted_score
,      ( select cast(collect( criteria||': '||score||' ('||weight||')') as string_table)
         from table(criteria)
         where score > 0) criteria
from   ( select empno
         ,      sum(score * weight) weighted_score
         ,      cast(collect(query_match_score_t( score, weight,criteria)) as query_match_score_table_t) criteria
         from   (
                 select empno
                 ,      0 score
                 ,      0 weight
                 ,      'base' criteria
                 from   emp e
                 union all
                 select empno
                 ,      number_range_score(sal, 1500, 1000, 2000) score
                 ,      2 weight
                 ,      'Salary must be as close to 1500 as possible, and at least between 1000 and 2000'  criteria
                 from   emp
                 union all
                 select empno
                 ,      number_range_score(extract (year from hiredate), 1982, 1982, 1983) score
                 ,      1 weight
                 ,      'Employee is ideally hired in 1982; a hiredate in 1983 is also somewhat valuable'
                 from   emp
                 union all
                 select empno
                 ,      case job when 'CLERK' then 1 when 'ANALYST' then 0.6 when 'SALESMAN' then 0.4 end  score
                 ,      1.5 weight
                 ,      'We prefer CLERKs; ANALYST scores 60% and SALESMAN is 40%'
                 from   emp
                )
         group
         by     empno
         order
         by     weighted_score desc
       ) scores
where  rownum < 6
